#define FORCE_IMPORT_ARRAY
#include <vortex-python/common.hpp>

extern void bind_core(py::module& m);

extern void bind_marker(py::module& m);
extern void bind_scan(py::module& m);

extern void bind_acquire(py::module& m);
extern void bind_format(py::module& m);
extern void bind_io(py::module& m);
extern void bind_process(py::module& m);

extern void bind_engine(py::module& m);
extern void bind_simple(py::module& m);
extern void bind_storage(py::module& m);

extern void bind_memory(py::module& m);

extern void bind_driver_alazar(py::module& m);
extern void bind_driver_daqmx(py::module& m);

PYBIND11_MODULE(vortex, m) {
    xt::import_numpy();

    bind_core(m);
    bind_marker(m);
    bind_scan(m);

    auto driver = m.def_submodule("driver");
    bind_driver_alazar(driver);
    bind_driver_daqmx(driver);

    bind_acquire(m);
    bind_process(m);
    bind_format(m);
    bind_io(m);

    bind_engine(m);
    bind_simple(m);
    bind_storage(m);

    bind_memory(m);
    
    m.attr("__version__") = VORTEX_VERSION;

    std::vector<std::string> feature;
#if defined(VORTEX_ENABLE_REFLEXXES)          
        feature.push_back("reflexxes");
#endif
#if defined(VORTEX_ENABLE_FFTW)
        feature.push_back("fftw");
#endif
#if defined(VORTEX_ENABLE_CUDA)
        feature.push_back("cuda");
#endif
#if defined(VORTEX_ENABLE_ALAZAR)
        feature.push_back("alazar");
#endif
#if defined(VORTEX_ENABLE_ALAZAR_GPU)
        feature.push_back("alazar_gpu");
#endif
#if defined(VORTEX_ENABLE_DAQMX)
        feature.push_back("daqmx");
#endif
#if defined(VORTEX_ENABLE_HDF5)
        feature.push_back("hdf5");
#endif
#if defined(VORTEX_EXCEPTION_GUARDS)
        feature.push_back("exception_guards");
#endif
#if defined(VORTEX_SERIALIZE_CUDA_KERNELS)
        feature.push_back("serial_cuda_kernels");
#endif
    m.attr("__feature__") = feature;

#if defined(VORTEX_SERIALIZE_CUDA_KERNELS)
    PyErr_WarnExplicit(PyExc_RuntimeWarning, "Vortex is compiled with CUDA kernel serialization enabled. This is a debug feature and may negatively impact performance.", VORTEX_PYTHON_EXTENSION_NAME, 0, "vortex", NULL);
#endif
#if !defined(VORTEX_EXCEPTION_GUARDS)
    PyErr_WarnExplicit(PyExc_RuntimeWarning, "Vortex is compiled with exception guards disabled. This is a debug feature. Exceptions raised during callbacks may cause program termination.", VORTEX_PYTHON_EXTENSION_NAME, 0, "vortex", NULL);
#endif
}
