import unittest
from math import log10

import cupy
from cupy.testing import assert_array_almost_equal

import pytest

from vortex.process import CUDAProcessor

@pytest.mark.skipif(int(cupy.cuda.Device().compute_capability) < 35, reason='GPU compute capability is insufficient')
class TestCudaProcessor(unittest.TestCase):
    def setUp(self):
        self.processor = CUDAProcessor()

        cfg = self.processor.config
        cfg.samples_per_record = 1376
        cfg.ascans_per_block = 250
        cfg.enable_log10 = False
        cfg.enable_ifft = False
        self.cfg = cfg

        self.spectra = cupy.zeros(cfg.input_shape, dtype=cupy.uint16)
        self.ascans = cupy.zeros(cfg.output_shape, dtype=cupy.int8)

    def test_copy(self):
        self.processor.initialize(self.cfg)

        self.spectra[:] = 1

        with self.assertRaises(AssertionError):
            assert_array_almost_equal(self.spectra, self.ascans)

        self.processor.next(self.spectra, self.ascans)

        assert_array_almost_equal(self.spectra, self.ascans)

    def test_ifft(self):
        self.cfg.enable_ifft = True
        self.processor.initialize(self.cfg)

        self.spectra[:] = 1
        self.processor.next(self.spectra, self.ascans)

        assert_array_almost_equal(cupy.fft.ifft(self.spectra, axis=1), self.ascans)

    def test_log10(self):
        self.cfg.enable_log10 = True
        self.processor.initialize(self.cfg)

        x = 100
        self.spectra[:] = x
        self.processor.next(self.spectra, self.ascans)

        assert_array_almost_equal(20*log10(x), self.ascans)

if __name__ == '__main__':
    unittest.main()
